import { watch } from 'vue';
import { useUid } from '../../../composition/useUid';
import { deepIterator } from '../../deep-iterator/deepIterator';
import { isField } from '../../type-guards/typeGuards';
function registerField(form, name, field) {
    var _a;
    const uid = useUid();
    const formField = form.registerField(uid, name, field.$value, (_a = field.$rules) !== null && _a !== void 0 ? _a : []);
    watch(formField.modelValue, () => {
        if (formField.touched) {
            form.validate(uid);
        }
    }, { deep: true });
    return {
        $uid: uid,
        $value: formField.modelValue,
        $errors: formField.errors,
        $hasError: formField.hasError,
        $validating: formField.validating,
        async $onBlur() {
            if (!formField.touched) {
                formField.touched = true;
                await form.validate(uid);
            }
        }
    };
}
export function transformFormData(form, data) {
    for (const [key, value, parent] of deepIterator(data)) {
        if (isField(value)) {
            const transformedField = registerField(form, key, value);
            parent[key] = transformedField;
        }
    }
}
