export class LinkedListNode {
    constructor(value) {
        this.next = null;
        this.prev = null;
        this.value = value;
    }
}
export class LinkedList {
    constructor() {
        this.head = null;
        this.tail = null;
        this.count = 0;
    }
    get first() {
        return this.head;
    }
    get last() {
        return this.tail;
    }
    addFirst(value) {
        const node = new LinkedListNode(value);
        if (this.count === 0) {
            this.head = node;
            this.tail = node;
        }
        else {
            node.next = this.head;
            this.head.prev = node;
            this.head = node;
        }
        this.count++;
        return node;
    }
    addLast(value) {
        const node = new LinkedListNode(value);
        if (this.count === 0) {
            this.head = node;
            this.tail = node;
        }
        else {
            node.prev = this.tail;
            this.tail.next = node;
            this.tail = node;
        }
        this.count++;
        return node;
    }
    remove(node) {
        if (this.count === 0) {
            return;
        }
        if (node === this.head) {
            this.removeFirst();
        }
        else if (node === this.tail) {
            this.removeLast();
        }
        else {
            node.prev.next = node.next;
            node.next.prev = node.prev;
            node.next = null;
            node.prev = null;
            this.count--;
        }
    }
    removeFirst() {
        if (this.count === 0) {
            return;
        }
        if (this.count === 1) {
            this.head = null;
            this.tail = null;
            this.count--;
        }
        else {
            this.head = this.head.next;
            this.head.prev.next = null;
            this.head.prev = null;
            this.count--;
        }
    }
    removeLast() {
        if (this.count === 0) {
            return;
        }
        if (this.count === 1) {
            this.head = null;
            this.tail = null;
            this.count--;
        }
        else {
            this.tail = this.tail.prev;
            this.tail.next.prev = null;
            this.tail.next = null;
            this.count--;
        }
    }
    valuesForwards() {
        let node = this.head;
        return {
            *[Symbol.iterator]() {
                for (; node !== null; node = node.next) {
                    yield node.value;
                }
            }
        };
    }
    valuesBackwards() {
        let node = this.tail;
        return {
            *[Symbol.iterator]() {
                for (; node !== null; node = node.prev) {
                    yield node.value;
                }
            }
        };
    }
}
