import { Ref, ComputedRef, UnwrapRef } from 'vue';
import { RefUnref } from '../types';
export declare type SimpleRule<T = any> = (value: T) => any;
export declare type KeyedRule = {
    key: string;
    rule?: (...values: any[]) => any;
};
export declare type Rule<T = any> = SimpleRule<T> | KeyedRule;
export declare type Field<TValue> = {
    $value: RefUnref<TValue>;
    $rules?: Rule<TValue extends any[] ? TValue : UnwrapRef<TValue>>[];
};
export declare type TransformedField<T> = {
    $uid: number;
    $value: T;
    $errors: string[];
    $hasError: boolean;
    $validating: boolean;
    $onBlur(): void;
};
export declare type TransformedFormData<T extends object> = T extends any ? {
    [K in keyof T]: T[K] extends Field<infer TValue> | undefined ? T[K] extends undefined ? undefined : TransformedField<UnwrapRef<TValue>> : T[K] extends object ? TransformedFormData<T[K]> : T[K];
} : never;
export declare type FormData<T extends object> = T extends any ? {
    [K in keyof T]: T[K] extends Field<infer TValue> | undefined ? UnwrapRef<TValue> : T[K] extends object ? FormData<T[K]> : T[K];
} : never;
export declare type FieldNames<T> = T extends Record<string, unknown> ? {
    [K in keyof T]-?: T[K] extends Field<any> | undefined ? K : FieldNames<T[K]>;
}[keyof T] : T extends (infer TArray)[] ? FieldNames<TArray> : never;
export declare type Keys = readonly (string | number)[];
export declare type DeepIndex<T, Ks extends Keys, R = unknown> = Ks extends [
    infer First,
    ...infer Rest
] ? First extends keyof T ? Rest extends Keys ? DeepIndex<T[First], Rest> : R : R : T;
declare type UseValidation<T extends object> = {
    form: TransformedFormData<T>;
    submitting: Ref<boolean>;
    errors: ComputedRef<string[]>;
    validateFields(names?: FieldNames<T>[]): Promise<FormData<T>>;
    resetFields(formData?: Partial<FormData<T>>): void;
    add<Ks extends Keys>(path: readonly [...Ks], value: DeepIndex<T, Ks> extends Array<infer TArray> ? TArray : DeepIndex<T, Ks>): void;
    remove(path: (string | number)[]): void;
};
/**
 *
 * @param formData The structure of your Form Data.
 * @description
 * Vue composition function for Form Validation.
 * @docs
 * https://github.com/JensDll/vue3-form-validation
 * @typescript
 * For better type inference, consider defining the structure
 * of your `formData` upfront and pass it as the generic parameter `T`:
 * ```
 * type FormData = {
 *   name: Field<string>,
 *   email: Field<string>,
 *   password: Field<string>
 * }
 *
 * const { ... } = useValidation<FormData>({ ... })
 * ```
 */
export declare function useValidation<T extends object>(formData: T): UseValidation<T>;
export {};
